#ifndef _LINUX_SUPERMOUNT_FS_H
#define _LINUX_SUPERMOUNT_FS_H

#include <linux/types.h>

#undef SUPERMOUNT_DEBUG

#define SUPERMOUNT_VERSION		"0.5"

/*
 * Debug code
 */
#ifdef SUPERMOUNT_DEBUG
	extern char supermount_debug_enable;
	#include <linux/delay.h>

	#define supermount_debug(f, a...) \
	{ \
		if (supermount_debug_enable) { \
			printk ("SUPERMOUNT DEBUG (%s:%s, %d): ", \
				__FILE__, __FUNCTION__, __LINE__); \
			printk (f, ## a); \
		} \
	}

	#define supermount_dump_inode(i) \
		supermount_debug("inode= %ld, sub=%ld, count=%d/%d, mode=%06o/%06o%s%s%s\n",	\
			 (i)->i_ino, 					\
			 ((i)->u.supermount_i.i_hidden ? 		\
			  (i)->u.supermount_i.i_hidden->i_ino : 0),	\
			 (i)->i_count,					\
			 ((i)->u.supermount_i.i_hidden ?		\
			  (i)->u.supermount_i.i_hidden->i_count : -1),	\
			 (i)->i_mode,					\
			 ((i)->u.supermount_i.i_hidden ?		\
			  (i)->u.supermount_i.i_hidden->i_mode : -1),	\
			 ((i)->i_state & I_DIRTY) ? ", dirty" : "",			\
			 inode_is_obsolete((i)) ? ", obsolete" : "",		\
			 ((i)->u.supermount_i.i_dentry) ? ", hidden dentry" : "")

	#define supermount_dump_normal_inode(i) \
		supermount_debug("inode= %ld, count=%d%s\n",	\
			 (i)->i_ino, 					\
			 (i)->i_count,					\
			 ((i)->i_state & I_DIRTY) ? ", dirty" : "");

#else
	#define supermount_debug(f, a...)	/**/
	#define supermount_dump_inode(i)
	#define supermount_dump_normal_inode(i)
#endif

/*
 * Special inodes numbers
 */
#define SUPERMOUNT_HIDDEN_INO		 0	/* Hidden inode for
										   sub-mounting */
#define SUPERMOUNT_ROOT_INO			 1	/* Root inode */

#ifdef __KERNEL__
/*
 * Function prototypes
 */

/*
 * Ok, these declarations are also in <linux/kernel.h> but none of the
 * supermount source programs needs to include it so they are duplicated here.
 */
# define NORET_TYPE    /**/
# define ATTRIB_NORET  __attribute__((noreturn))
# define NORET_AND     noreturn,

/* How to test if a supermount filesystem is active or not: */

static inline int subfs_is_active(struct super_block *sb)
{
	/*
	 * The subfs is always active if it is write-accessed.  (We
	 * don't resort to inode counts in that case since we can be
	 * write-active even if we are only accessing directories, in
	 * the case of such operations as renames.)  Otherwise, it is
	 * deemed inactive iff only the root is open, and its i_count is
	 * precisely one.  
	 */
	return sb->u.supermount_sb.s_writecount ||
		(sb->u.supermount_sb.s_state != SUPERMOUNT_UNMOUNTED &&
		 (sb->u.supermount_sb.s_opencount > 1/* ||
		  sb->u.supermount_sb.s_undermount->d_mounts->d_inode->i_count > 1*/));
}
/* How to test if an inode is obsolete: */
static inline int inode_is_obsolete(struct inode *inode)
{
	return (inode->u.supermount_i.i_sb_version !=
		inode->i_sb->u.supermount_sb.s_version);
}
/* Manage the subfs dirty flag */
static inline void mark_subfs_dirty(struct super_block *sb)
{
	sb->u.supermount_sb.s_dirty = 1;
}
static inline void mark_subfs_clean(struct super_block *sb)
{
	sb->u.supermount_sb.s_dirty = 0;
}
static inline int subfs_is_dirty(struct super_block *sb)
{
	return (sb->u.supermount_sb.s_dirty);
}

/* shadow.c */
extern void supermount_read_hidden_inode (struct inode *);

/* inode.c */
extern struct inode * subiget(struct inode *);
extern void supermount_iopen(struct super_block *);
extern void supermount_iclose(struct super_block *);
extern struct inode * supermount_attach(struct super_block *, struct dentry *);
extern void supermount_read_inode (struct inode *);
extern void supermount_write_inode (struct inode *);
extern void supermount_put_inode (struct inode *);
extern int supermount_notify_change(struct dentry *, struct iattr *);
extern int supermount_permission (struct inode * inode, int mask);

/* namei.c */
extern struct dentry *supermount_lookup (struct inode *, struct dentry *);
extern int supermount_create (struct inode *, struct dentry *, int);
extern int supermount_unlink (struct inode *, struct dentry *);
extern int supermount_mkdir (struct inode *, struct dentry *, int);
extern int supermount_rmdir (struct inode *, struct dentry *);
extern int supermount_mknod (struct inode *, struct dentry *, int, int);
extern int supermount_symlink (struct inode *, struct dentry *, const char *);
extern int supermount_rename (struct inode *, struct dentry *,
							  struct inode *, struct dentry *);
#if 0
extern int supermount_link (struct inode *, struct inode *, const char *, int);
#endif /* 0 */
/* super.c */
extern void supermount_error (struct super_block *, const char *, const char *, ...)
	__attribute__ ((format (printf, 3, 4)));
extern NORET_TYPE void supermount_panic (struct super_block *, const char *,
				   const char *, ...)
	__attribute__ ((NORET_AND format (printf, 3, 4)));
extern void supermount_warning (struct super_block *, const char *, const char *, ...)
	__attribute__ ((format (printf, 3, 4)));
extern void supermount_put_super (struct super_block *);
extern void supermount_write_super (struct super_block *);
extern int supermount_revalidate_inode(struct super_block *sb, struct inode *inode);
#if 0
extern int supermount_remount (struct super_block *, int *, char *);
#endif /* 0 */
extern struct super_block * supermount_read_super (struct super_block *,void *,int);
extern int supermount_statfs (struct super_block *, struct statfs *, int);
extern int supermount_media_check (struct super_block *);
#if 0
extern int supermount_check_inode (struct inode *);
#endif /* 0 */
extern void supermount_lock_door (struct super_block *, int);
#if 0
extern int supermount_test_write_access (struct super_block *);
#endif /* 0 */
extern int supermount_get_write_access (struct super_block *);
extern void supermount_put_write_access (struct super_block *);
extern void supermount_go_inactive(struct super_block *);

/* dir.c */
extern struct inode_operations supermount_dir_iops;
extern struct dentry_operations supermount_dir_dops;
/* inode.c */
extern struct inode_shadow_operations supermount_shadow_iops;
/* file.c */
extern struct dentry_operations supermount_file_dops;
#if 0
/* symlink.c */
extern struct inode_operations supermount_symlink_iops;
#endif /* 0 */
#endif	/* __KERNEL__ */

#endif	/* _LINUX_SUPERMOUNT_FS_H */
